var AclEngine = function() {
	var READ_ONLY = "R";
	var INVISIBLE = "I";

	var extendRules = function (component, parentComponent) {
		component.aclIsVisible = parentComponent.aclIsVisible;
		component.aclReadOnly = parentComponent.aclReadOnly;
	};

	var setDefaultRule = function(component){
		component.aclReadOnly = true;
		component.aclIsVisible = false;
	};

	var increaseRuleOfParent = function (component, parentComponent) {
		if (parentComponent !== null && component.aclIsVisible) {
			parentComponent.aclReadOnly = false;
			parentComponent.aclIsVisible = true;
		}
	};

	var setAclRules = function (component, parentComponent, aclRules, userGroupId) {
		var aclRule = aclRules[component.id] || aclRules[component.name];
		var rule = (aclRule && aclRule.userGroups[userGroupId]) ?
			aclRule.userGroups[userGroupId].rule : undefined;
		/*Novas regras do controle de acesso
		if(rule){
			component.aclReadOnly = (rule === READ_ONLY);
			component.aclIsVisible = (rule !== INVISIBLE);
		} else if(parentComponent){
			extendRules(component, parentComponent);
		} else {
			setDefaultRule(component);
		}*/
		if (!rule && parentComponent) {
			extendRules(component, parentComponent);
		} else {
			component.aclReadOnly = (rule === READ_ONLY);
			component.aclIsVisible = (rule !== INVISIBLE);
		}
	};

	var _processMenuAclContainerReadOnly = function (menu, containers) {
		if (menu.aclReadOnly && menu.windowName && containers) {
			if (!containers[menu.windowName]) {
				containers[menu.windowName] = {};
			}
			containers[menu.windowName].isAccessible = false;
		}
	};

	var _processMenuAclMenuChildren = function (menu, aclRules, containers, userGroupId) {
		if (menu.menus && menu.menus.length > 0) {
			var childrenMaxRule = _processMenuAcl(menu.menus, menu, aclRules, containers, userGroupId);
			increaseRuleOfParent(childrenMaxRule, menu);
		}
	};

	var _processMenuAcl = function (menus, parentMenu, aclRules, containers, userGroupId) {
		var maxRules = {};
		menus.forEach(function(menu, index) {
			setAclRules(menu, parentMenu, aclRules, userGroupId);
			_processMenuAclMenuChildren(menu, aclRules, containers, userGroupId);
			increaseRuleOfParent(menu, maxRules);
			_processMenuAclContainerReadOnly(menu, containers);
		});
		return maxRules;
	};

	this.processMenuAcl = function (menus, parentMenu, aclRules, containers, userGroupId) {
		if (aclRules && Array.isArray(menus)) {
			_processMenuAcl(menus, parentMenu, aclRules, containers, userGroupId);
		}
	};

	var getComponentChildren = function (component) {
		var children = [];
		if (component.widgets) {
			children = children.concat(component.widgets);
		}
		if (component.fields) {
			children = children.concat(component.fields);
		}
		if (component.actions) {
			children = children.concat(component.actions);
		}
		if (component.notifications) {
			children = children.concat(component.notifications);
		}

		return children;
	};

	var _processAclChildren = function (component, aclRules, userGroupId) {
		var children = getComponentChildren(component);
		if (children) {
			var maxRules = {};
			for (var i in children) {
				if (children.hasOwnProperty(i)) {
					var childrenRule = _processAcl(children[i], component, aclRules, userGroupId);
					increaseRuleOfParent(childrenRule, maxRules);
				}
			}
			increaseRuleOfParent(maxRules, component);
		}
	};

	var _processAcl = function (component, parentComponent, aclRules, userGroupId) {
		setAclRules(component, parentComponent, aclRules, userGroupId);
		_processAclChildren(component, aclRules, userGroupId);
		return component;
	};

	this.processAcl = function (component, parentComponent, aclRules, userGroupId) {
		if (aclRules && component) {
			_processAcl(component, parentComponent, aclRules, userGroupId);
		}
	};
};